#!/usr/bin/env python
# -*- coding: utf-8 -*-

from os import system

import pygtk
pygtk.require("2.0")
import gtk

from cgapwddialog import pwdDialog
from cgapwdchangedialog import pwdChangeDialog
from cgapwdmanager import pwdManager
from cgaconfig import *

class editFilterWidget(gtk.Window):
    def __init__ (self, friendly_name):
        """ Edita un fichero de texto. Se provee un nombre amigable para usar como título de la ventana y un fichero a editar """
        super(editFilterWidget, self).__init__(gtk.WINDOW_TOPLEVEL)
        self.set_icon_from_file("/usr/share/pixmaps/cga-filtro-edu20.svg")

        self.pm=pwdManager()

        accelgroup = gtk.AccelGroup()
        key, modifier = gtk.accelerator_parse('Escape')
        accelgroup.connect_group(key, modifier, gtk.ACCEL_VISIBLE, gtk.main_quit)
        self.add_accel_group(accelgroup)

        self.connect("delete_event", gtk.main_quit)

        self.fn=friendly_name
        self.modificado = {"blanca": False, "negra": False}
        self.color_rojo = gtk.gdk.Color(255, 0, 0)
        self.color_verde = gtk.gdk.Color(0, 255, 0)

        layout = gtk.VBox(homogeneous=False, spacing=5)

        # Botón aceptar y señales
        self.b_aceptar=gtk.Button(gtk.STOCK_OK)
        self.b_aceptar.set_use_stock(True)
        self.b_aceptar.connect("clicked", self.aceptar)

        # Botón cancelar y señales
        self.b_cancelar=gtk.Button(gtk.STOCK_CANCEL)
        self.b_cancelar.set_use_stock(True)
        self.b_cancelar.connect("clicked", self.confirmar_salir)

        # Botón aplicar y señales
        self.b_aplicar = gtk.Button(gtk.STOCK_APPLY)
        self.b_aplicar.set_use_stock(True)
        self.b_aplicar.connect("clicked", self.aplicar)

        # Botón de autorizar
        self.b_auth=gtk.Button("A_utorizar")
        self.b_auth.connect("clicked", self.autorizar)
        self.b_auth.set_flags(gtk.CAN_DEFAULT)
        self.set_default(self.b_auth)

        # Botón de cambio de contraseña
        self.b_change_pwd = gtk.Button("Ca_mbiar contraseña")
        self.b_change_pwd.connect("clicked", self.change_pwd)

        # Botón de habilitar/deshabilitar filtrado
        self.b_enabled = gtk.Button("Habilitar")
        self.b_enabled.connect("clicked", self.enable_disable_privoxy)

        # Letrero de estado del filtro.
        self.e_estado = gtk.Label("Estado del filtro:")
        self.e_estado.set_use_markup(True)

        estado_box = gtk.HBox()
        estado_box.pack_start(self.e_estado)

        # Botonera de autorización
        botonera_auth=gtk.HButtonBox()
        botonera_auth.add(self.b_auth)
        botonera_auth.add(self.b_change_pwd)
        botonera_auth.add(self.b_enabled)

        # Botonera inferior
        botonera = gtk.HButtonBox()
        botonera.add(self.b_cancelar)
        botonera.add(self.b_aplicar)
        botonera.add(self.b_aceptar)
#        botonera.set_layout(gtk.BUTTONBOX_END)

        ### Lista negra
        # Contenido del textview
        self.contenido_negro=gtk.TextBuffer()
        # El textview que muestra el textbuffer
        self.t_edicion_negro = gtk.TextView()
        self.t_edicion_negro.set_buffer(self.contenido_negro)
        # El scroll para poder hacer idem en el texto
        scrolled_negra=gtk.ScrolledWindow()
        scrolled_negra.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_negra.add(self.t_edicion_negro)

        # Elegante marco que rodea el texto
        frame1=gtk.Frame()
        frame1.add(scrolled_negra)

        ### Lista blanca
        # Contenido del textview
        self.contenido_blanco=gtk.TextBuffer()
        # El textview que muestra el textbuffer
        self.t_edicion_blanco = gtk.TextView()
        self.t_edicion_blanco.set_buffer(self.contenido_blanco)
        # El scroll para poder hacer idem en el texto
        scrolled_blanca=gtk.ScrolledWindow()
        scrolled_blanca.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_blanca.add(self.t_edicion_blanco)

        # Elegante marco que rodea el texto
        frame2=gtk.Frame()
        frame2.add(scrolled_blanca)

        # Widget de tabs
        tabs=gtk.Notebook()
        l_blanca=gtk.Label("Lista blanca")
        l_negra=gtk.Label("Lista negra")

        tabs.append_page(frame1, l_negra)
        tabs.append_page(frame2, l_blanca)

        # Arriba el "editor" y abajo los botones.
        layout.pack_start(tabs, True, True, 5)
        layout.pack_start(estado_box, False, False, 5)
        layout.pack_start(botonera_auth, False, True, 5)
        layout.pack_start(botonera, False, True, 5)

        # Morralla estandar
        self.add(layout)
        self.resize(320, 450)
        self.set_position(gtk.WIN_POS_CENTER)
        self.set_title(friendly_name)

        self.set_autorizado(False)

        self.show_all()

    def set_autorizado(self, autorizado):
        self.autorizado=autorizado
        self.update_ui()

    def update_ui(self):
        l_widgets=[ self.b_aceptar, self.b_aplicar, self.b_change_pwd, self.t_edicion_negro, self.t_edicion_blanco, self.b_enabled ]

        for w in l_widgets:
            w.set_sensitive(self.autorizado)

        if self.autorizado:
            self.load_listas()
            self.contenido_negro.place_cursor(self.contenido_negro.get_start_iter())
            self.contenido_negro.connect("changed", self.modificar, "negra")
            self.contenido_blanco.place_cursor(self.contenido_blanco.get_start_iter())
            self.contenido_blanco.connect("changed", self.modificar, "blanca")

        self.b_auth.set_sensitive(not self.autorizado)

        # Establece el estado del botón "Filtrado activado/desactivado"
        try:
            with open(PRIVOXY_CGACONFIG, "r") as f:
                for linea in f.readlines():
                    linea=linea.strip().split("=")
                    if linea[0] == "PRIVOXY_RUN":
                        self.privoxy_enabled = linea[1] == "yes"
        except IOError:
            # El fichero no existe, se deja como activado
            self.privoxy_enabled = True

        self.update_enable_disable_button()

    def update_enable_disable_button(self):
        if self.privoxy_enabled:
            self.b_enabled.set_label("Desactivar filtrado")
        else:
            self.b_enabled.set_label("Activar filtrado")

        formato = "<span weight='bold' foreground='%s'>%s</span>" % [("red", "Desactivado"),("#008532", "Activado")][self.privoxy_enabled]
        self.e_estado.set_label("Estado del filtro: %s" % formato)

    def enable_disable_privoxy(self, boton):
        self.privoxy_enabled = not self.privoxy_enabled
        self.update_enable_disable_button()
        self.update_privoxy_config_file()

    def update_privoxy_config_file(self):
        estado = ["no", "yes"][self.privoxy_enabled]
        with open(PRIVOXY_CGACONFIG, "w") as f:
            f.write("PRIVOXY_RUN=%s" % estado)
        self.restart_service()

    def load_listas(self):
        try:
            blanco=open(WHITELIST, "r").read()
        except IOError:
            blanco=""
        try:
            negro=open(BLACKLIST, "r").read()
        except IOError:
            negro=""
    
        self.contenido_blanco.set_text(blanco)
        self.contenido_negro.set_text(negro)

    def modificar(self, w, cual):
        self.modificado[cual]=True

    def save_file(self):
        """ Guarda la plantilla con las listas configuradas por el usuario """
        datos = [(self.contenido_negro,  PLANTILLA_BLACK, CONF_BLACK, BLACKLIST ),
                 (self.contenido_blanco, PLANTILLA_WHITE, CONF_WHITE, WHITELIST ) ]

        for (w, plantilla, conf, cache)  in datos:
            cont=w.get_text(w.get_start_iter(),w.get_end_iter())
            header=open(plantilla, "r").read()
            with open(conf, "w") as f:
                f.write(header)
                f.write("\n")
                f.write(cont)
            with open(cache, "w") as f:
                f.write(cont)

        self.modificado["blanca"]=self.modificado["negra"]=False

    def aplicar(self, param=None):
        if any(self.modificado.values()):
            self.save_file()
            self.restart_service()

    def aceptar(self, _):
        self.aplicar()
        gtk.main_quit()

    def confirmar_salir(self, *args):
        if any(self.modificado.values()):
            self.pregunta_salir()
        else:
            gtk.main_quit()

    def pregunta_salir(self):
        mensaje="Hay cambios sin guardar, ¿Seguro que desea salir?"
        m=gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, gtk.BUTTONS_YES_NO, mensaje)
        res=m.run()
        m.hide()
        if res == gtk.RESPONSE_YES:
            gtk.main_quit()
        
    def restart_service(self):
        print "Reiniciando servicio"
        comando="/etc/init.d/privoxy-cgaconfig restart"
        ret=system(comando)
        if ret==0:
            print "Éxito reiniciando privoxy"
        else:
            print "No se pudo reiniciar privoxy"

    def autorizar(self, widget):
        d=pwdDialog(parent=self)
        res=d.run()
        if res == gtk.RESPONSE_OK:
            if self.pm.check(d.get_pass()):
                self.autorizado=True
                self.update_ui()
            else:
                m=gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Contraseña erronea")
                res=m.run()
                m.hide()
        d.hide()

    def change_pwd(self, widget):
        p=pwdChangeDialog(parent=self)
        res=p.run()
        if res== gtk.RESPONSE_OK:
            if p.iguales():
                self.pm.update(p.get_pass())
                self.pm.save()

                m=gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, "Contraseña cambiada correctamente.")
                res=m.run()
                m.hide()
            else:
                m=gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Las constraseñas no coinciden.")
                res=m.run()
                m.hide()
        p.hide()

if __name__== "__main__":
    c=editFilterWidget("Edición de filtro de accesos")
    gtk.main()
